@@ -1,7 +1,7 @@
 # This file is part of Tryton.  The COPYRIGHT file at the top level of
 # this repository contains the full copyright notices and license terms.
-from decimal import Decimal
 from collections import defaultdict
+from decimal import Decimal
 
 from sql import Column, Literal, Null
 from sql.aggregate import Sum
@@ -9,12 +9,14 @@ from sql.conditionals import Coalesce
 
 from trytond import backend
 from trytond.i18n import gettext
-from trytond.model import (ModelView, ModelSQL, DeactivableMixin, fields,
-    Unique, tree)
-from trytond.wizard import Wizard, StateView, StateAction, Button
-from trytond.pyson import Eval, If, PYSONEncoder, PYSONDecoder
-from trytond.transaction import Transaction
+from trytond.model import (
+    DeactivableMixin, ModelSQL, ModelView, Unique, fields, tree)
+from trytond.modules.currency.fields import Monetary
 from trytond.pool import Pool
+from trytond.pyson import Eval, If, PYSONDecoder, PYSONEncoder
+from trytond.tools import lstrip_wildcard
+from trytond.transaction import Transaction
+from trytond.wizard import Button, StateAction, StateView, Wizard
 
 from .exceptions import AccountValidationError
 
@@ -30,8 +32,6 @@ class Account(
     currency = fields.Function(
         fields.Many2One('currency.currency', 'Currency'),
         'on_change_with_currency')
-    currency_digits = fields.Function(fields.Integer('Currency Digits'),
-        'on_change_with_currency_digits')
     type = fields.Selection([
         ('root', 'Root'),
         ('view', 'View'),
@@ -47,8 +47,7 @@ class Account(
         states={
             'invisible': Eval('type') == 'root',
             'required': Eval('type') != 'root',
-            },
-        depends=['company', 'type'])
+            })
     parent = fields.Many2One('analytic_account.account', 'Parent', select=True,
         domain=['OR',
             ('root', '=', Eval('root', -1)),
@@ -57,30 +56,28 @@ class Account(
         states={
             'invisible': Eval('type') == 'root',
             'required': Eval('type') != 'root',
-            },
-        depends=['root', 'type'])
+            })
     childs = fields.One2Many('analytic_account.account', 'parent', 'Children',
         states={
             'invisible': Eval('id', -1) < 0,
             },
         domain=[
             ('company', '=', Eval('company', -1)),
-            ],
-        depends=['company'])
-    balance = fields.Function(fields.Numeric('Balance',
-        digits=(16, Eval('currency_digits', 1)), depends=['currency_digits']),
+            ])
+    balance = fields.Function(Monetary(
+            "Balance", currency='currency', digits='currency'),
         'get_balance')
-    credit = fields.Function(fields.Numeric('Credit',
-        digits=(16, Eval('currency_digits', 2)), depends=['currency_digits']),
+    credit = fields.Function(Monetary(
+            "Credit", currency='currency', digits='currency'),
         'get_credit_debit')
-    debit = fields.Function(fields.Numeric('Debit',
-        digits=(16, Eval('currency_digits', 2)), depends=['currency_digits']),
+    debit = fields.Function(Monetary(
+            "Debit", currency='currency', digits='currency'),
         'get_credit_debit')
     state = fields.Selection([
         ('draft', 'Draft'),
         ('opened', 'Opened'),
         ('closed', 'Closed'),
-        ], 'State', required=True)
+        ], "State", required=True, sort=False)
     note = fields.Text('Note')
     distributions = fields.One2Many(
         'analytic_account.account.distribution', 'parent',
@@ -88,8 +85,7 @@ class Account(
         states={
             'invisible': Eval('type') != 'distribution',
             'required': Eval('type') == 'distribution',
-            },
-        depends=['type'])
+            })
     distribution_parents = fields.Many2Many(
         'analytic_account.account.distribution', 'account', 'parent',
         "Distribution Parents", readonly=True)
@@ -124,30 +120,27 @@ class Account(
         return 'draft'
 
     @classmethod
-    def validate(cls, accounts):
-        super(Account, cls).validate(accounts)
-        for account in accounts:
-            account.check_distribution()
+    def validate_fields(cls, accounts, field_names):
+        super().validate_fields(accounts, field_names)
+        cls.check_distribution(accounts, field_names)
 
-    def check_distribution(self):
-        if self.type != 'distribution':
+    @classmethod
+    def check_distribution(cls, accounts, field_names=None):
+        if field_names and not (field_names & {'distributions', 'type'}):
             return
-        if sum((d.ratio for d in self.distributions)) != 1:
-            raise AccountValidationError(
-                gettext('analytic_account.msg_invalid_distribution',
-                    account=self.rec_name))
+        for account in accounts:
+            if account.type != 'distribution':
+                return
+            if sum((d.ratio for d in account.distributions)) != 1:
+                raise AccountValidationError(
+                    gettext('analytic_account.msg_invalid_distribution',
+                        account=account.rec_name))
 
     @fields.depends('company')
     def on_change_with_currency(self, name=None):
         if self.company:
             return self.company.currency.id
 
-    @fields.depends('company')
-    def on_change_with_currency_digits(self, name=None):
-        if self.company:
-            return self.company.currency.digits
-        return 2
-
     @fields.depends('parent', 'type',
         '_parent_parent.id', '_parent_parent.root', '_parent_parent.type')
     def on_change_parent(self):
@@ -191,7 +184,7 @@ class Account(
                 & (table.active == Literal(True)) & line_query,
                 group_by=table.id))
         account_sum = defaultdict(Decimal)
-        for account_id, value in cursor.fetchall():
+        for account_id, value in cursor:
             account_sum.setdefault(account_id, Decimal('0.0'))
             # SQLite uses float for SUM
             if not isinstance(value, Decimal):
@@ -206,8 +199,7 @@ class Account(
                     ])
             for child in childs:
                 balance += account_sum[child.id]
-            exp = Decimal(str(10.0 ** -account.currency_digits))
-            balances[account.id] = balance.quantize(exp)
+            balances[account.id] = account.currency.round(balance)
         return balances
 
     @classmethod
@@ -245,7 +237,7 @@ class Account(
                 & (table.active == Literal(True)) & line_query,
                 group_by=table.id))
 
-        for row in cursor.fetchall():
+        for row in cursor:
             account_id = row[0]
             for i, name in enumerate(names, 1):
                 value = row[i]
@@ -255,9 +247,8 @@ class Account(
                 result[name][account_id] += value
         for account in accounts:
             for name in names:
-                exp = Decimal(str(10.0 ** -account.currency_digits))
-                result[name][account.id] = (
-                    result[name][account.id].quantize(exp))
+                result[name][account.id] = account.currency.round(
+                    result[name][account.id])
         return result
 
     def get_rec_name(self, name):
@@ -268,11 +259,17 @@ class Account(
 
     @classmethod
     def search_rec_name(cls, name, clause):
-        accounts = cls.search([('code',) + tuple(clause[1:])], limit=1)
-        if accounts:
-            return [('code',) + tuple(clause[1:])]
+        if clause[1].startswith('!') or clause[1].startswith('not '):
+            bool_op = 'AND'
         else:
-            return [(cls._rec_name,) + tuple(clause[1:])]
+            bool_op = 'OR'
+        code_value = clause[2]
+        if clause[1].endswith('like'):
+            code_value = lstrip_wildcard(clause[2])
+        return [bool_op,
+            ('code', clause[1], code_value) + tuple(clause[3:]),
+            (cls._rec_name,) + tuple(clause[1:]),
+            ]
 
     def distribute(self, amount):
         "Return a list of (account, amount) distribution"
@@ -340,8 +337,7 @@ class AccountDistribution(ModelView, ModelSQL):
         'analytic_account.account', "Account", required=True,
         domain=[
             ('root', '=', Eval('root', -1)),
-            ],
-        depends=['root'])
+            ])
     ratio = fields.Numeric("Ratio", required=True,
         domain=[
             ('ratio', '>=', 0),
@@ -371,15 +367,13 @@ class AnalyticAccountEntry(ModelView, ModelSQL):
                 (),
                 ('company', '=', Eval('company', -1))),
             ('type', '=', 'root'),
-            ],
-        depends=['company'])
+            ])
     account = fields.Many2One('analytic_account.account', 'Account',
         ondelete='RESTRICT',
         domain=[
             ('root', '=', Eval('root')),
             ('type', 'in', ['normal', 'distribution']),
-            ],
-        depends=['root', 'company'])
+            ])
     company = fields.Function(fields.Many2One('company.company', 'Company'),
         'on_change_with_company', searcher='search_company')
 
@@ -387,7 +381,9 @@ class AnalyticAccountEntry(ModelView, ModelSQL):
     def __register__(cls, module_name):
         pool = Pool()
         Account = pool.get('analytic_account.account')
-        cursor = Transaction().connection.cursor()
+        transaction = Transaction()
+        cursor = transaction.connection.cursor()
+        update = transaction.connection.cursor()
 
         # Migration from 3.4: use origin as the key for One2Many
         migration_3_4 = False
@@ -407,8 +403,8 @@ class AnalyticAccountEntry(ModelView, ModelSQL):
             cursor.execute(*account.select(account.id, account.root,
                     where=account.type != 'root'))
             entry = cls.__table__()
-            for account_id, root_id in cursor.fetchall():
-                cursor.execute(*entry.update(
+            for account_id, root_id in cursor:
+                update.execute(*entry.update(
                         columns=[entry.root],
                         values=[root_id],
                         where=entry.account == account_id))
@@ -431,11 +427,9 @@ class AnalyticAccountEntry(ModelView, ModelSQL):
     @classmethod
     def get_origin(cls):
         Model = Pool().get('ir.model')
+        get_name = Model.get_name
         models = cls._get_origin()
-        models = Model.search([
-                ('model', 'in', models),
-                ])
-        return [(None, '')] + [(m.model, m.name) for m in models]
+        return [(None, '')] + [(m, get_name(m)) for m in models]
 
     def on_change_with_company(self, name=None):
         return None
@@ -465,8 +459,7 @@ class AnalyticMixin(object):
     __slots__ = ()
     analytic_accounts = fields.One2Many('analytic.account.entry', 'origin',
         'Analytic Accounts',
-        size=Eval('analytic_accounts_size', 0),
-        depends=['analytic_accounts_size'])
+        size=Eval('analytic_accounts_size', 0))
     analytic_accounts_size = fields.Function(fields.Integer(
             'Analytic Accounts Size'), 'get_analytic_accounts_size')
 
@@ -474,7 +467,9 @@ class AnalyticMixin(object):
     def __register__(cls, module_name):
         pool = Pool()
         AccountEntry = pool.get('analytic.account.entry')
-        cursor = Transaction().connection.cursor()
+        transaction = Transaction()
+        cursor = transaction.connection.cursor()
+        update = transaction.connection.cursor()
 
         super(AnalyticMixin, cls).__register__(module_name)
 
@@ -485,8 +480,8 @@ class AnalyticMixin(object):
             table = cls.__table__()
             cursor.execute(*table.select(table.id, table.analytic_accounts,
                     where=table.analytic_accounts != Null))
-            for line_id, selection_id in cursor.fetchall():
-                cursor.execute(*entry.update(
+            for line_id, selection_id in cursor:
+                update.execute(*entry.update(
                         columns=[entry.origin],
                         values=['%s,%s' % (cls.__name__, line_id)],
                         where=entry.selection == selection_id))
