@@ -1,27 +1,29 @@
 # This file is part of Tryton.  The COPYRIGHT file at the top level of
 # this repository contains the full copyright notices and license terms.
-from decimal import Decimal
 from collections import defaultdict
+from decimal import Decimal
 from itertools import groupby
 
 from sql import Literal
 
-from trytond.model import ModelView, ModelSQL, fields, Check
-from trytond.wizard import Wizard, StateAction
+from trytond.model import Check, ModelSQL, ModelView, fields
+from trytond.modules.currency.fields import Monetary
+from trytond.pool import Pool, PoolMeta
 from trytond.pyson import Eval, PYSONEncoder
 from trytond.transaction import Transaction
-from trytond.pool import Pool, PoolMeta
+from trytond.wizard import StateAction, Wizard
 
 
 class Line(ModelSQL, ModelView):
     'Analytic Line'
     __name__ = 'analytic_account.line'
-    debit = fields.Numeric('Debit', digits=(16, Eval('currency_digits', 2)),
-        required=True, depends=['currency_digits'])
-    credit = fields.Numeric('Credit', digits=(16, Eval('currency_digits', 2)),
-        required=True, depends=['currency_digits'])
-    currency_digits = fields.Function(fields.Integer('Currency Digits'),
-        'on_change_with_currency_digits')
+    debit = Monetary(
+        "Debit", currency='currency', digits='currency', required=True)
+    credit = Monetary(
+        "Credit", currency='currency', digits='currency', required=True)
+    currency = fields.Function(fields.Many2One(
+            'currency.currency', "Currency"),
+        'on_change_with_currency')
     company = fields.Function(fields.Many2One('company.company', 'Company'),
         'on_change_with_company', searcher='search_company')
     account = fields.Many2One('analytic_account.account', 'Account',
@@ -31,8 +33,7 @@ class Line(ModelSQL, ModelView):
                 ('company', '=', None),
                 ('company', '=', Eval('company', -1)),
                 ],
-            ],
-        depends=['company'])
+            ])
     move_line = fields.Many2One('account.move.line', 'Account Move Line',
             ondelete='CASCADE', required=True)
     date = fields.Date('Date', required=True)
@@ -74,10 +75,9 @@ class Line(ModelSQL, ModelView):
         return Decimal(0)
 
     @fields.depends('move_line', '_parent_move_line.account')
-    def on_change_with_currency_digits(self, name=None):
+    def on_change_with_currency(self, name=None):
         if self.move_line and self.move_line.account:
-            return self.move_line.account.company.currency.digits
-        return 2
+            return self.move_line.account.company.currency.id
 
     @fields.depends('move_line', '_parent_move_line.account')
     def on_change_with_company(self, name=None):
@@ -175,7 +175,7 @@ class MoveLine(ModelSQL, ModelView):
     analytic_state = fields.Selection([
             ('draft', 'Draft'),
             ('valid', 'Valid'),
-            ], 'Analytic State', readonly=True, select=True)
+            ], "Analytic State", readonly=True, select=True, sort=False)
 
     @classmethod
     def __setup__(cls):
@@ -276,7 +276,7 @@ class MoveLine(ModelSQL, ModelView):
             for analytic_line in line.analytic_lines:
                 amount = analytic_line.debit - analytic_line.credit
                 amounts[analytic_line.account.root] += amount
-            roots = company2roots[line.move.company]
+            roots = company2roots.get(line.move.company, set())
             if not roots <= set(amounts.keys()):
                 line.analytic_state = 'draft'
                 continue
